/* 
   $Id: sgaddr_btn.c,v 1.4 2001/12/25 10:43:56 sho Exp $

   Copyright (C) 2001,2002 AXE,Inc. <sg@axe-inc.co.jp>

   This file is part of sgaddr.

   sgaddr is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   sgaddr is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with sgaddr; see the file COPYING.   If not, write to
   the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <X11/X.h>
#include <gdk/gdk.h>
#include <gdk/gdkkeysyms.h>

#include "sgaddr.h"

extern GtkWidget  *notebook;
extern GtkWidget  *name_entry, *ruby_entry, *home_entry;
extern GtkWidget  *phone_entry, *mobile_entry, *email_entry, *etc_entry;
extern GtkWidget  *list;
extern GtkWidget  *main_note, *main_text;
extern int        array_tail, current_num;

int  current_state = STATE_REEDIT;

void set_edit_state(int state)
{
  current_state = state;
}

void press_prev_btn()
{
  current_num--;
  if(current_num < 0) current_num = 0;

  if(array_tail > 0)
    set_person_data_to_entry(get_person_data(current_num));    

  set_entry_editable(FALSE);
  set_edit_state(STATE_REEDIT);
}

void press_next_btn()
{
  current_num++;
  if(current_num >= array_tail) current_num = array_tail - 1;

  if(array_tail > 0)
    set_person_data_to_entry(get_person_data(current_num));    

  set_entry_editable(FALSE);
  set_edit_state(STATE_REEDIT);
}

void press_add_btn()
{
  struct person_data *tmp = NULL;

  tmp = (struct person_data *) malloc(sizeof(struct person_data));
  bzero(tmp, sizeof(struct person_data));

  tmp->name   = get_string_from_entry(name_entry, tmp->name);
  tmp->ruby   = get_string_from_entry(ruby_entry, tmp->ruby);
  tmp->home   = get_string_from_entry(home_entry, tmp->home);
  tmp->phone  = get_string_from_entry(phone_entry, tmp->phone);
  tmp->mobile = get_string_from_entry(mobile_entry, tmp->mobile);
  tmp->email  = get_string_from_entry(email_entry, tmp->email);
  tmp->etc    = get_string_from_entry(etc_entry, tmp->etc);

  set_entry_editable(FALSE);

  insert_person_data(tmp);
  save_person_data();

  sort_by_ruby();

  set_person_data_all(GTK_CLIST(list));
  set_edit_state(STATE_REEDIT);
}

void close_dialog(GtkWidget *widget, gpointer data)
{
  gtk_grab_remove(widget);
}

void
message_btn(char *message, void (*y_func)(), void (*n_func)())
{
  GtkWidget *dialog;
  GtkWidget *d_mesg;
  GtkWidget *y_btn, *n_btn;

  dialog = gtk_dialog_new();

  gtk_signal_connect(GTK_OBJECT(dialog), "destroy",
		     GTK_SIGNAL_FUNC(close_dialog), NULL);

  gtk_window_set_title(GTK_WINDOW(dialog), NULL);

  d_mesg = gtk_label_new(message);

  gtk_misc_set_padding(GTK_MISC(d_mesg), 10, 10);

  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox),
		     d_mesg, TRUE, TRUE, 0);

  y_btn = gtk_button_new_with_label(_("Yes"));
  n_btn = gtk_button_new_with_label(_("No"));

  gtk_signal_connect(GTK_OBJECT(y_btn), "clicked",
		     GTK_SIGNAL_FUNC(y_func), dialog);
  gtk_signal_connect(GTK_OBJECT(n_btn), "clicked",
		     GTK_SIGNAL_FUNC(n_func), dialog);

  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area),
		     y_btn, TRUE, TRUE, 0);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area),
		     n_btn, TRUE, TRUE, 0);

  gtk_widget_show(d_mesg);
  gtk_widget_show(y_btn);
  gtk_widget_show(n_btn);
  gtk_widget_show(dialog);
  gtk_grab_add(dialog);
}

void yes_delete(GtkWidget *widget, gpointer data)
{
  if(array_tail < 1) return;

  remove_person_data(current_num);

  current_num--;
  if(current_num < 0) current_num = 0;

  sort_by_ruby();
  set_person_data_all(GTK_CLIST(list));

  if(array_tail > 0)
    set_person_data_to_entry(get_person_data(current_num));    

  set_edit_state(STATE_REEDIT);
  gtk_widget_destroy(GTK_WIDGET(data));
}

void no_delete(GtkWidget *widget, gpointer data)
{
  gtk_widget_destroy(GTK_WIDGET(data));
}

void press_del_btn()
{
  message_btn(_("Do you want to delete?"), yes_delete, no_delete);
}

void press_reedit_btn()
{
  set_entry_editable(TRUE);
  set_edit_state(STATE_REEDIT | STATE_EDITING);
}

void press_new_btn()
{
  set_entry_editable(TRUE);
  set_string_to_entry(name_entry, NULL);
  set_string_to_entry(ruby_entry, NULL);
  set_string_to_entry(home_entry, NULL);
  set_string_to_entry(phone_entry, NULL);
  set_string_to_entry(mobile_entry, NULL);
  set_string_to_entry(email_entry, NULL);
  set_string_to_entry(etc_entry, NULL);
  set_edit_state(STATE_NEWENTRY | STATE_EDITING);

  gtk_notebook_set_page(GTK_NOTEBOOK(notebook), 1);
}

void press_apply_btn()
{
  struct person_data *current;

  if(!(current_state & STATE_REEDIT)) return;

  if(array_tail < 1) return;

  current = get_person_data(current_num);    

  current->name   = get_string_from_entry(name_entry,   current->name);
  current->ruby   = get_string_from_entry(ruby_entry,   current->ruby);
  current->home   = get_string_from_entry(home_entry,   current->home);
  current->phone  = get_string_from_entry(phone_entry,  current->email);
  current->mobile = get_string_from_entry(mobile_entry, current->mobile);
  current->email  = get_string_from_entry(email_entry,  current->email);
  current->etc    = get_string_from_entry(etc_entry,    current->etc);

  set_entry_editable(FALSE);

  sort_by_ruby();
  set_person_data_all(GTK_CLIST(list));

  set_edit_state(STATE_REEDIT);
}

void button_press(GtkWidget *widget, gpointer data)
{
  switch((int)data){
  case BTN_PREV:  press_prev_btn();  break;
  case BTN_NEXT:  press_next_btn();  break;
  case BTN_REEDIT:  press_reedit_btn();  break;
  case BTN_APPLY: press_apply_btn(); break;
  case BTN_NEW:   press_new_btn();   break;
  case BTN_ADD:   press_add_btn();   break;
  case BTN_DEL:   press_del_btn();   break;
  }
}

int entry_num = 0;

void sgaddr_main_button_press(GtkWidget *widget, gpointer data)
{
  char   *p = NULL;

  switch((int)data){
  case ADDR_NAME:   p = gtk_entry_get_text(GTK_ENTRY(name_entry));   break;
  case ADDR_RUBY:   p = gtk_entry_get_text(GTK_ENTRY(ruby_entry));   break;
  case ADDR_HOME:   p = gtk_entry_get_text(GTK_ENTRY(home_entry));   break;
  case ADDR_PHONE:  p = gtk_entry_get_text(GTK_ENTRY(phone_entry));  break;
  case ADDR_MOBILE: p = gtk_entry_get_text(GTK_ENTRY(mobile_entry)); break;
  case ADDR_EMAIL:  p = gtk_entry_get_text(GTK_ENTRY(email_entry));  break;
  case ADDR_ETC:    p = gtk_entry_get_text(GTK_ENTRY(etc_entry));    break;
  }
  entry_num = (int)data;
  gtk_editable_delete_text(GTK_EDITABLE(main_text), 0, -1);
  gtk_text_insert(GTK_TEXT(main_text), NULL, NULL, NULL, p, strlen(p));
  if(current_state & STATE_EDITING)
    gtk_text_set_editable(GTK_TEXT(main_text), TRUE);

  gtk_notebook_set_page(GTK_NOTEBOOK(main_note), 1);
}

void
sgaddr_return_button(GtkWidget *widget, gpointer data)
{
  char *p = NULL;

  p = gtk_editable_get_chars(GTK_EDITABLE(main_text), 0,
			     gtk_text_get_length(GTK_TEXT(main_text)));
  switch(entry_num){
  case ADDR_NAME:    set_string_to_entry(name_entry, p);   break;
  case ADDR_RUBY:    set_string_to_entry(ruby_entry, p);   break;
  case ADDR_HOME:    set_string_to_entry(home_entry, p);   break;
  case ADDR_PHONE:   set_string_to_entry(phone_entry, p);  break;
  case ADDR_MOBILE:  set_string_to_entry(mobile_entry, p); break;
  case ADDR_EMAIL:   set_string_to_entry(email_entry, p);  break; 
  case ADDR_ETC:     set_string_to_entry(etc_entry, p);    break;
  }
  if(!(current_state & STATE_EDITING))  
    gtk_text_set_editable(GTK_TEXT(main_text), FALSE);
  gtk_notebook_set_page(GTK_NOTEBOOK(main_note), 0);
}
